from unittest.mock import patch

import pytest

from gso.products import Iptrunk
from test.workflows import (
    assert_complete,
    assert_suspended,
    extract_state,
    resume_workflow,
    run_workflow,
    user_accept_and_assert_suspended,
)


@pytest.mark.workflow
@patch("gso.workflows.iptrunk.modify_isis_metric.provisioning_proxy.provision_ip_trunk")
def test_iptrunk_modify_isis_metric_success(
    mock_provision_ip_trunk,
    iptrunk_subscription_factory,
    faker,
):
    #  Set up mock return values
    product_id = iptrunk_subscription_factory()
    new_isis_metric = faker.pyint()

    #  Run workflow
    initial_iptrunk_data = [
        {"subscription_id": product_id},
        {"tt_number": faker.tt_number(), "isis_metric": new_isis_metric},
    ]
    result, process_stat, step_log = run_workflow("modify_isis_metric", initial_iptrunk_data)
    assert_suspended(result)

    lso_return = {
        "pp_run_results": {
            "status": "ok",
            "job_id": faker.uuid4(),
            "output": "parsed_output",
            "return_code": 0,
        },
        "confirm": "ACCEPTED",
    }

    result, step_log = user_accept_and_assert_suspended(process_stat, step_log, lso_return)
    result, step_log = user_accept_and_assert_suspended(process_stat, step_log, [{}, {}])
    result, step_log = user_accept_and_assert_suspended(process_stat, step_log, lso_return)
    result, step_log = resume_workflow(process_stat, step_log, [{}, {}])

    assert_complete(result)

    state = extract_state(result)
    subscription_id = state["subscription_id"]
    subscription = Iptrunk.from_subscription(subscription_id)

    assert "active" == subscription.status
    assert mock_provision_ip_trunk.call_count == 2
    assert subscription.iptrunk.iptrunk_isis_metric == new_isis_metric
